import ohh
import sys
import getpass
import os
import requests
import json
import dateutil.parser
from ohh.background_runner import BackgroundRunner
from datetime import datetime
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


class Runner(object):
    def __init__(self, basedir='~/.ohh'):
        super(Runner, self).__init__()
        self._basedir = os.path.expanduser(basedir)

        if not os.path.isdir(self._basedir):
            os.mkdir(self._basedir)

        self._log_file = os.path.join(self._basedir, 'ohh.log')
        self._pid_file = os.path.join(self._basedir, 'ohh.pid')
        self._config_file = os.path.join(self._basedir, 'ohhrc')
        self._info_file = os.path.join(self._basedir, 'session_info')

        self.config = ConfigParser()
        self._read_config()
        self._background_runner = BackgroundRunner(self._pid_file)

    def _read_input(self):
        python_version = sys.version_info.major
        if python_version == 3:
            return input()
        else:
            return raw_input()

    def _create_config(self):
        self.config.add_section('ohh')
        for k, v in ohh.DEFAULT_CONFIG.items():
            self.config.set('ohh', k, v)
        self._save_config()

    def _read_config(self):
        if not os.path.isfile(self._config_file):
            self._create_config()
        else:
            self.config.read(self._config_file)

    def _post(self, path, data=None):
        headers = {}
        if data:
            data = json.dumps(data)
            headers['Content-Type'] = 'application/json'
        base_url = self.config.get('ohh', 'endpoint')
        url = os.path.join(base_url, path)
        token = self.config.get('ohh', 'token')
        if token:
            headers['X-Token'] = token
        return requests.post(url, data=data, headers=headers)

    def login(self):
        sys.stdout.write('Email: ')
        email = self._read_input()
        password = getpass.getpass()
        r = self._post('authenticate', {'mail': email, 'password': password})
        if r.status_code == 401:
            raise ohh.OhhError('wrong email or password')
        self._check_http_status('login', r.status_code)
        self.config.set('ohh', 'token', r.json()['token'])
        self._save_config()

    def _save_config(self):
        with open(self._config_file, 'w') as f:
            self.config.write(f)

    def start(self, start_daemon=True):
        if self.is_running():
            print("Session already started, use 'ohh stop' to stop.")
            return
        self._check_credentials()
        r = self._post('sessions/start')
        self._check_http_status('start', r.status_code)
        self._write_info('active')
        if start_daemon and not self.is_running():
            self._background_runner.start(self, self._basedir)

    def _write_info(self, status):
        with open(self._info_file, 'w') as f:
            f.write(json.dumps({
                'start_time': datetime.now().isoformat(),
                'status': status,
            }))

    def stop(self, kill_daemon=True):
        self._check_credentials()
        r = self._post('sessions/end')
        self._check_http_status('stop', r.status_code)
        if os.path.isfile(self._info_file):
            os.remove(self._info_file)
        if kill_daemon and self.is_running():
            self._background_runner.stop()

    def is_running(self):
        pid = self._background_runner.get_pid()
        return pid and os.path.exists('/proc/%d' % pid)

    def _check_http_status(self, action, status_code):
        if status_code != 200:
            msg = 'failed to {0}, got status {1}'
            raise ohh.OhhError(msg.format(action, status_code))

    def status(self):
        if self.is_running():
            pid = self._background_runner.get_pid()
            print('ohh is running (pid {0})'.format(pid))
            self.show_info()
        else:
            print('ohh is not running')

    def show_info(self):
        if not os.path.isfile(self._info_file):
            return
        with open(self._info_file, 'r') as f:
            status = json.loads(f.read())
        timediff = self._timediff(status['start_time'])
        print("You have been {0} for {1}.".format(status['status'], timediff))

    def _check_credentials(self):
        token = self.config.get('ohh', 'token')
        if token is None or token == 'None':
            raise ohh.OhhError("Please run 'ohh login'")

    def _timediff(self, start_time):
        start_time = dateutil.parser.parse(start_time)
        s = (datetime.now() - start_time).seconds
        if s < 60:
            return '{0} seconds'.format(s)
        elif s < 3600:
            return '{0} minutes and {1} seconds'.format(s // 60, s % 60)
        else:
            return '{0} hours and {1} minutes'.format(s // 3600, s // 60 % 60)

    def run(self, action):
        return getattr(self, action)()



"""ohh - CLI tool for OHH project"""

from ohh.runner import Runner

DEFAULT_CONFIG = {
    'endpoint': 'https://ohh-api.claudetech.com',
    'token': 'None',
}


class OhhError(Exception):
    def __init__(self, message):
        self.message = message


__version__ = '0.1.8'
__author__ = 'Daniel Perez <daniel@claudetech.com>'
__all__ = []



from ohh.daemon import Daemon
import signal
import psutil
from os.path import join
import sys
import collections
import logging
import json
from logging.handlers import RotatingFileHandler


class BackgroundRunner(Daemon):
    def run(self, runner, basedir):
        self._make_logger(basedir)
        self._basedir = basedir
        self._status = 'active'
        self._runner = runner
        self._usage_data = collections.deque(maxlen=60*5)
        signal.signal(signal.SIGINT, self._handle_stop)
        while True:
            cpu_usage = psutil.cpu_percent(interval=1)
            self._usage_data.append(cpu_usage)
            idle = self._is_idle()
            if idle and self._status == 'active':
                self._change_status('idle')
            elif not idle and self._status == 'idle':
                self._change_status('active')

    def _make_logger(self, basedir):
        logfile = join(basedir, 'ohh.log')
        self._logger = logging.getLogger('ohh')
        handler = RotatingFileHandler(logfile, maxBytes=1024**2)
        self._logger.addHandler(handler)

    def _change_status(self, status):
        if status == 'active':
            self._runner.start(start_daemon=False)
        else:
            self._runner.stop(kill_daemon=False)
        self._usage_data.clear()
        self._status = status
        self._logger.log('status changed to {0}'.format(self._status))
        self._runner._write_info(self._status)

    # TODO: tune logic
    def _is_idle(self):
        if len(self._usage_data) < 60*5:
            return False
        total = 0
        high_usages_count = 0
        for percent in self._usage_data:
            if percent > 10:
                high_usages_count += 1
            total += percent
        return high_usages_count < 30

    def _handle_stop(self, signal, frame):
        self._runner.stop(kill_daemon=False)
        sys.exit(0)

if __name__ == '__main__':
    from ohh.runner import Runner
    import os
    runner = Runner()
    bg_runner = BackgroundRunner('/tmp/ohh.pid')
    bg_runner.run(runner, os.path.expanduser('~/.ohh'))



import argparse
from ohh import __version__

actions = ['start', 'stop', 'login', 'status']


def make_parser():
    parser = argparse.ArgumentParser(description='ohh CLI')
    parser.add_argument('action',
                        metavar='action',
                        type=str,
                        help='the action to execute. available actions: ' + ', '.join(actions),
                        choices=actions)
    parser.add_argument('--basedir',
                        metavar='BASE_DIR',
                        type=str,
                        default='~/.ohh',
                        help='directory containing ohh data')
    parser.add_argument('--version', action='version',
                        version='%(prog)s {version}'.format(version=__version__))
    return parser


def parse():
    parser = make_parser()
    return parser.parse_args()



import setuptools

setuptools.setup(
    name="ohh",
    version="0.1.8",
    url="https://git.claudetech.com/ohh/ohh-cli",

    author="Daniel Perez",
    author_email="daniel@claudetech.com",

    description="CLI tool for OHH project",
    long_description=open('README.md').read(),

    packages=setuptools.find_packages(),

    scripts=['bin/ohh'],

    install_requires=[
        'requests',
        'psutil',
        'python-dateutil',
    ],

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
)
